/*===================================================================*/
//-------------------         Configuration       -------------------//
//-------------------        for first time       -------------------//
/*===================================================================*/
var mraa = require('mraa');
var myOnboardLed = new mraa.Gpio(13);
myOnboardLed.dir(mraa.DIR_OUT);
var lamp1 = new mraa.Gpio(2);
lamp1.dir(mraa.DIR_OUT);
var lamp2 = new mraa.Gpio(3);
lamp2.dir(mraa.DIR_OUT);
var valve = new mraa.Gpio(4);
valve.dir(mraa.DIR_OUT);
var fan = new mraa.Gpio(5);
fan.dir(mraa.DIR_OUT);
var upmBuzzer = require("jsupm_buzzer");
var myBuzzer = new upmBuzzer.Buzzer(6);
var grove_moisture = require('jsupm_grovemoisture');
var moisture1 = new grove_moisture.GroveMoisture(0);
var moisture2 = new grove_moisture.GroveMoisture(1);
var groveSensor = require('jsupm_grove');
var light1 = new groveSensor.GroveLight(4);
var light2 = new groveSensor.GroveLight(5);
var upmTP401 = require('jsupm_gas');
var airSensor = new upmTP401.TP401(2);
var LM35 = new mraa.Aio(0);

/*===================================================================*/
//-------------------       Onboard LED blink     -------------------//
//-------------------           PIN D13            -------------------//
/*===================================================================*/
var ledState;
function periodicActivity() {
    ledState = myOnboardLed.read();                  // get the current state of the LED pin
    myOnboardLed.write(ledState?0:1);                // if the pin is currently 1 write a '0' (low) else write a '1' (high)
    setTimeout(periodicActivity,100);
};
periodicActivity();  // start the periodic toggle


/*===================================================================*/
//-------------------         Lamp Relay x2       -------------------//
//-------------------    Lamp1: D2, Lamp2: D3     -------------------//
/*===================================================================*/
var lamp1State;
function lamp1_on() {
  lamp1.write(1);
  lamp1State = 1;
  process.stdout.write('lamp1 is on\r\n') ;    // and write the lamp1 state to the console
};

function lamp1_off() {
  lamp1.write(0);
  lamp1State = 0;
  process.stdout.write('lamp1 is off\r\n') ;    // and write the lamp1 state to the console
};

function Lamp1Period() {
    lamp1_on();
    setTimeout(lamp1_off, 60000);    // Testing On 1 minutes, real time is On 16 Hours = 57600000 ms 
    setTimeout(Lamp1Period, 300000);  // Testing Off 4 minutes, real time is Off 8 Hours = 86400000 ms
};

lamp1_off();
Lamp1Period() ; // start the periodic toggle




var lamp2State;
function lamp2_on(){
  lamp2.write(1);
  lamp2State = 1;
  process.stdout.write('lamp2 is on\r\n') ;    // and write the lamp2 state to the console
};

function lamp2_off(){
  lamp2.write(0);
  lamp2State = 0;
  process.stdout.write('lamp2 is off\r\n') ;    // and write the lamp2 state to the console
};

function Lamp2Period() {
    lamp2_on();
    setTimeout(lamp2_off, 60000);    // Testing On 1 minutes, real time is On 16 Hours = 57600000 ms 
    setTimeout(Lamp2Period, 300000);  // Testing Off 4 minutes, real time is Off 8 Hours = 86400000 ms
};

lamp2_off();
Lamp2Period() ; // start the periodic toggle


/*===================================================================*/
//-------------------       Valve & Fan Relay     -------------------//
//-------------------      Valve: D4, Fan: D5     -------------------//
/*===================================================================*/
var valveState;
function valve_on(){
  valve.write(0);
  valveState = 1;
  process.stdout.write('valve is on\r\n') ;    // and write the valve state to the console
};

function valve_off(){
  valve.write(1);
  valveState = 0;
  process.stdout.write('valve is off\r\n') ;    // and write the valve state to the console
};

var valvePeriod = function() {
    valve_on();
    setTimeout(valve_off, 1000);
};

valve_off();
valvePeriod();



var fanState;
function fan_on(){
  fan.write(0);
  fanState = 1;
  process.stdout.write('fan is on\r\n') ;    // and write the fan state to the console
};

function fan_off(){
  fan.write(1);
  fanState = 0;
  process.stdout.write('fan is off\r\n') ;    // and write the fan state to the console
};

function fanPeriod() {
    fan_on();
    setTimeout(fan_off, 20000);   // Testing On 20 Seconds, real time is On 30 Seconds
    setTimeout(fanPeriod, 60000); // Testing Period is 60 Seconds, real time is 1 Hours = 3600000 ms 
};

fan_off();
fanPeriod();



/*===================================================================*/
//-------------------           Speaker           -------------------//
//-------------------        Connect to D6        -------------------//
/*===================================================================*/
var chords = [];
chords.push(upmBuzzer.DO);
chords.push(upmBuzzer.RE);
chords.push(upmBuzzer.MI);
chords.push(upmBuzzer.FA);
chords.push(upmBuzzer.SOL);
chords.push(upmBuzzer.LA);
chords.push(upmBuzzer.SI);
var chordIndex = 0;

// Print sensor name
console.log(myBuzzer.name());

function melody()
{
    if (chords.length != 0)
    {
        //Play sound for one second
        console.log( myBuzzer.playSound(chords[chordIndex], 200) );
        chordIndex++;
        //Reset the sound to start from the beginning. 
        if (chordIndex > chords.length - 1)
			chordIndex = 0;
    }
}

function beep()
{
    myBuzzer.playSound(chords[5], 200);
    
}

melody();
beep();

// Print message when exiting
process.on('SIGINT', function()
{
	console.log("Exiting...");
	process.exit(0);
});


/*===================================================================*/
//-------------------       Water flow sensor     -------------------//
//-------------------             Pin D7          -------------------//
/*===================================================================*/

var waterFlow_lib = require('jsupm_grovewfs');
// Instantiate a Grove Water Flow Sensor on digital pin D7
var WaterFlow = new waterFlow_lib.GroveWFS(7);

// set the flow counter to 0 and start counting
WaterFlow.clearFlowCounter();
WaterFlow.startFlowCounter();


var millis, flowCount, fr;
var myInterval = setInterval(function()
{
	// we grab these (millis and flowCount) just for display
	// purposes in this example
	millis = WaterFlow.getMillis();
	flowCount = WaterFlow.flowCounter();

	fr = WaterFlow.flowRate();

	// output milliseconds passed, flow count, and computed flow rate
	outputStr = "Millis: " + millis + " Flow Count: " + flowCount +
	" Flow Rate: " + fr + " LPM";
	console.log(outputStr);

	// best to gather data for at least one second for reasonable
	// results.
}, 2000);


// When exiting: clear interval and print message
process.on('SIGINT', function()
{
	clearInterval(myInterval);
	WaterFlow.stopFlowCounter();
	WaterFlow = null
	waterFlow_lib.cleanUp();
	waterFlow_lib = null;

	console.log("Exiting");
	process.exit(0);
});


/*===================================================================*/
//-------------------        Moisture sensor      -------------------//
//------------------- Sensor 1: A0, Sensor 2: A1  -------------------//
/*===================================================================*/
var moisture_val1;
var moisture_val2;
// Values (approximate):
// 0-300,   sensor in air or dry soil
// 300-600, sensor in humid soil
// 600+,    sensor in wet soil or submerged in water
// Read the value every second and print the corresponding moisture level
function moisture_value() {
  var result1;
	moisture_val1 = parseInt(moisture1.value());
  moisture_val2 = parseInt(moisture2.value());
	if (moisture_val1 >= 0 && moisture_val1 < 300)
		result1 = "Dry";
	else if (moisture_val1 >= 300 && moisture_val1 < 600)
		result1 = "Moist";
	else
		result1 = "Wet";
	console.log("Moisture 1 value: " + moisture_val1 + ", " + result1);

  if (moisture_val2 >= 0 && moisture_val2 < 300)
		result2 = "Dry";
	else if (moisture_val2 >= 300 && moisture_val2 < 600)
		result2 = "Moist";
	else
		result2 = "Wet";
	console.log("Moisture 2 value: " + moisture_val2 + ", " + result2);
  setTimeout(moisture_value,1000);
};

moisture_value();


/*===================================================================*/
//-------------------         Light sensor        -------------------//
//------------------- Sensor 1: A4, Sensor 2: A5  -------------------//
/*===================================================================*/
var light_raw1;
var light_raw2;
var light_lux1;
var light_lux2;
// Read the input and print both the raw value and a rough lux value,
// waiting one second between readings
function readLightSensorValue() {
  light_raw1 = light1.raw_value();
  light_raw2 = light2.raw_value();
  light_lux1 = light1.value();
  light_lux2 = light2.value();
  console.log(light1.name() + " raw value is " + light_raw1 +
            ", which is roughly " + light_lux1 + " lux" + '\r\n');
  console.log(light2.name() + " raw value is " + light_raw2 +
            ", which is roughly " + light_lux2 + " lux" + '\r\n');
  setTimeout(readLightSensorValue,1000);
}

readLightSensorValue();

/*===================================================================*/
//-------------------      Air quality sensor     -------------------//
//-------------------     Sensor connect to A2    -------------------//
/*===================================================================*/
var air_value;
var air_ppm;
//give a qualitative meaning to the value from the sensor
function airQuality(value)
{
    if(value < 50) return "Fresh Air";
    if(value < 200) return "Normal Indoor Air";
    if(value < 400) return "Low Pollution";
    if(value < 600) return "High Pollution - Action Recommended";
    return "Very High Pollution - Take Action Immediately";
}

function air_value()
{
    //read values (consecutive reads might vary slightly)
    air_value = airSensor.getSample();
    air_ppm = airSensor.getPPM();

    //write the sensor values to the console
    console.log("raw: " + air_value + " ppm: " + (" " + air_ppm.toFixed(2)).substring(-5, 5) + "   " + airQuality(air_value));

    //wait 2.5 s then call function again
    setTimeout(air_value, 1000);
}

air_value();


/*===================================================================*/
//-------------------   LM35 Temperature sensor   -------------------//
//-------------------     Sensor connect to A3    -------------------//
/*===================================================================*/
var temp_val;
function getTemperature() {
  var counts = LM35.read();
  var volts = counts * 495 / 1023;
  temp_val = parseFloat(volts).toFixed(4);
  setTimeout(getTemperature,1000);
};

getTemperature();


/*===================================================================*/
//----------------------     IBM Watson     -------------------------//
//----------------------        mqtt        -------------------------//
/*===================================================================*/

var ORG = 'ldms21';
var TYPE = 'IntelEdison';
var ID = 'EdisonGarden';
var AUTHTOKEN = '80p+KfglQUddFH_X@T';

var mqtt    = require('mqtt');

var PROTOCOL = 'mqtt';
var BROKER = ORG + '.messaging.internetofthings.ibmcloud.com';
var PORT = 1883;

//Create the url string
var URL = PROTOCOL + '://' + BROKER;
URL += ':' + PORT; 
//URL is e.g. 'mqtt://xrxlila.messaging.internetofthings.ibmcloud.com:1883'

var CLIENTID= 'd:' + ORG;
CLIENTID += ':' + TYPE;
CLIENTID += ':' + ID;
//CLIENTID -s e.g. d:xrxila:edison-air:784b87a801e9

var AUTHMETHOD = 'use-token-auth';

var client  = mqtt.connect(URL, { clientId: CLIENTID, username: AUTHMETHOD, password: AUTHTOKEN });

var TOPIC = 'iot-2/evt/status/fmt/json';
console.log(TOPIC);

client.on('connect', function () {
 setInterval(function(){
 //client.publish(TOPIC, '{"d":{ "Volts":' + analogVolts() + ' ,"Temperature":' + getTemperature() + '}}');//Payload is JSON
 client.publish(TOPIC, '{"d":{ "Lamp1":' + lamp1State + ' ,"Lamp2":' + lamp2State + ' ,"Valve":' + valveState + ' ,"Fan":' + fanState + ' ,"Moisture1":' + moisture_val1 + ' ,"Moisture2":' + moisture_val2 + ' ,"LightSensor1":' + light_lux1 + ' ,"LightSensor2":' + light_lux2 + ' ,"AirQuality":' + air_ppm + ' ,"Temp":' + temp_val + '}}');//Payload is JSON
 }, 2000);//Keeps publishing every 2000 milliseconds.
});



/*===================================================================*/
//-------------------            UART0            -------------------//
//-------------------         CNC Garden          -------------------//
/*===================================================================*/

Motor_shield = new mraa.Uart(0); //Default
var serialPath = Motor_shield.getDevicePath();
var SerialPort = require('serialport');
var serialPort = new SerialPort(serialPath, {
    baudrate: 57600
});
serialPort.on("open",function() {
    console.log("open");
    console.log("Connected to "+serialPath);
    serialPort.on("data", function(data) { //Read available data from serial port
    console.log("data received: " + data);
    });
    serialPort.write("G00 X2000 Y2000 F100;\r\n", function(err, results) { //Write data
        //console.log("err " + err);
        //console.log("results " + results);
    });
});



/*===================================================================*/
//-------------------     WebCam Realtime     -----------------------//
//-------------------         local           -----------------------//
/*===================================================================*/

// modules
var childProcess = require('child_process')
  , express = require('express')
  , http = require('http')
  , morgan = require('morgan')
  , ws = require('ws');

// configuration files
var configServer = require('./lib/config/server');

// app parameters
var app = express();
app.set('port', configServer.httpPort);
app.use(express.static(configServer.staticFolder));
app.use(morgan('dev'));

// serve index
require('./lib/routes').serveIndex(app, configServer.staticFolder);

// HTTP server
http.createServer(app).listen(app.get('port'), function () {
  console.log('HTTP server listening on port ' + app.get('port'));
});

/// Video streaming section
// Reference: https://github.com/phoboslab/jsmpeg/blob/master/stream-server.js

var STREAM_MAGIC_BYTES = 'jsmp'; // Must be 4 bytes
var width = 320;
var height = 240;

// WebSocket server
var wsServer = new (ws.Server)({ port: configServer.wsPort });
console.log('WebSocket server listening on port ' + configServer.wsPort);

wsServer.on('connection', function(socket) {
  // Send magic bytes and video size to the newly connected socket
  // struct { char magic[4]; unsigned short width, height;}
  var streamHeader = new Buffer(8);

  streamHeader.write(STREAM_MAGIC_BYTES);
  streamHeader.writeUInt16BE(width, 4);
  streamHeader.writeUInt16BE(height, 6);
  socket.send(streamHeader, { binary: true });

  console.log('New WebSocket Connection (' + wsServer.clients.length + ' total)');

  socket.on('close', function(code, message){
    console.log('Disconnected WebSocket (' + wsServer.clients.length + ' total)');
  });
});

wsServer.broadcast = function(data, opts) {
  for(var i in this.clients) {
    if(this.clients[i].readyState == 1) {
      this.clients[i].send(data, opts);
    }
    else {
      console.log('Error: Client (' + i + ') not connected.');
    }
  }
};

// HTTP server to accept incoming MPEG1 stream
http.createServer(function (req, res) {
  console.log(
    'Stream Connected: ' + req.socket.remoteAddress +
    ':' + req.socket.remotePort + ' size: ' + width + 'x' + height
  );

  req.on('data', function (data) {
    wsServer.broadcast(data, { binary: true });
	res.write('video');
  });
}).listen(configServer.streamPort, function () {
  console.log('Listening for video stream on port ' + configServer.streamPort);

  // Run do_ffmpeg.sh from node                                                   
  childProcess.exec('../../bin/do_ffmpeg.sh');
});

module.exports.app = app;
